import aiohttp
import asyncio
import logging

# Setup logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class AsyncTelegramClient:
    def __init__(self, token):
        self.base_url = f"https://api.telegram.org/bot{token}/"
        self.session = aiohttp.ClientSession()
        self.offset = None

    async def listen_for_messages(self, handler):
        while True:
            try:
                updates = await self.get_updates(self.offset)
                for update in updates.get('result', []):
                    chat_id = update['message']['chat']['id']
                    text = update['message'].get('text', '')
                    await handler(self, chat_id, text)
                    self.offset = update['update_id'] + 1
            except Exception as e:
                logger.error(f"Error in listen_for_messages: {e}")
                await asyncio.sleep(1)

    async def send_message(self, chat_id, text, parse_mode=None, reply_markup=None):
        try:
            url = self.base_url + "sendMessage"
            data = {
                "chat_id": chat_id,
                "text": text,
                "parse_mode": parse_mode,
                "reply_markup": reply_markup
            }
            async with self.session.post(url, data=data) as response:
                return await response.json()
        except Exception as e:
            logger.error(f"Error in send_message: {e}")

    async def get_updates(self, offset=None):
        try:
            url = self.base_url + "getUpdates"
            params = {"offset": offset} if offset else {}
            async with self.session.get(url, params=params) as response:
                return await response.json()
        except Exception as e:
            logger.error(f"Error in get_updates: {e}")
            return {}

    async def close(self):
        await self.session.close()

async def runbot(token, message_handler):
    client = AsyncTelegramClient(token)
    await client.listen_for_messages(message_handler)
    await client.close()



import re
import html

# Regular expressions for Markdown entities
BOLD_RE = re.compile(r"\*\*(.*?)\*\*")
ITALIC_RE = re.compile(r"__(.*?)__")
CODE_RE = re.compile(r"`(.*?)`")
URL_RE = re.compile(r"\[(.*?)\]\((.*?)\)")

# Simple function to escape HTML
def escape_html(text):
    return html.escape(text)

def parse_markdown(text):
    """Converts Markdown text to HTML."""
    # Escape HTML
    text = escape_html(text)
    
    # Bold
    text = BOLD_RE.sub(r"<b>\1</b>", text)
    
    # Italic
    text = ITALIC_RE.sub(r"<i>\1</i>", text)
    
    # Inline code
    text = CODE_RE.sub(r"<code>\1</code>", text)
    
    # URLs
    text = URL_RE.sub(r'<a href="\2">\1</a>', text)
    
    return text

def unparse_html(text):
    """Converts HTML back to Markdown. This is a simplified approach."""
    # URLs
    text = re.sub(r'<a href="(.*?)">(.*?)</a>', r'[\2](\1)', text)
    
    # Inline code
    text = text.replace("<code>", "`").replace("</code>", "`")
    
    # Italic
    text = text.replace("<i>", "__").replace("</i>", "__")
    
    # Bold
    text = text.replace("<b>", "**").replace("</b>", "**")
    
    return text



import requests
import json
from .markdown import parse_markdown, unparse_html

class TelegramClient:
    def __init__(self, token):
        self.base_url = f"https://api.telegram.org/bot{token}/"
        self.offset = None
        self.command_handlers = {}  # Dictionary to store command handlers
        self.openai_api_key = None
        self.openai_model = "gpt-3.5-turbo"

    def enable_openai_integration(self, openai_api_key, openai_model="gpt-3.5-turbo"):
        """Optionally enable OpenAI integration by providing an API key."""
        self.openai_api_key = openai_api_key
        self.openai_model = openai_model
        try:
            import openai
            openai.api_key = self.openai_api_key
            self.openai_available = True
        except ImportError:
            self.openai_available = False
            print("OpenAI library is not installed. Please install openai to use this feature.")

    def listen_for_messages(self, handler):
        while True:
            updates = self.get_updates(self.offset)
            for update in updates.get('result', []):
                chat_id = update['message']['chat']['id']
                text = update['message'].get('text', '')
                handler(self, chat_id, text)
                self.offset = update['update_id'] + 1

    def send_message(self, chat_id, text, parse_mode=None, reply_markup=None):
        url = self.base_url + "sendMessage"
        data = {"chat_id": chat_id, "text": text, "parse_mode": parse_mode, "reply_markup": reply_markup}
        if parse_mode not in ["Markdown", "MarkdownV2", "HTML", None]:
            raise ValueError(f"Unsupported parse_mode: {parse_mode}")
            response = requests.post(url, json=data)
            print(response.text)  # To debug the response from Telegram API

    
    def send_photo(self, chat_id, photo, caption=None, parse_mode=None):
        url = self.base_url + "sendPhoto"
        data = {"chat_id": chat_id, "caption": caption, "parse_mode": parse_mode}
        files = {'photo': open(photo, 'rb')}  # Ensure 'photo' is the path to the photo file
        requests.post(url, data=data, files=files)


    def send_audio(self, chat_id, audio, caption=None, parse_mode=None):
        url = self.base_url + "sendAudio"
        data = {"chat_id": chat_id, "caption": caption, "parse_mode": parse_mode}
        files = {'audio': open(audio, 'rb')}
        requests.post(url, data=data, files=files)

    def send_video(self, chat_id, video, caption=None, parse_mode=None):
        url = self.base_url + "sendVideo"
        data = {"chat_id": chat_id, "caption": caption, "parse_mode": parse_mode}
        files = {'video': open(video, 'rb')}
        requests.post(url, data=data, files=files)

    def send_document(self, chat_id, document, caption=None, parse_mode=None):
        url = self.base_url + "sendDocument"
        data = {"chat_id": chat_id, "caption": caption, "parse_mode": parse_mode}
        files = {'document': open(document, 'rb')}
        requests.post(url, data=data, files=files)

    def add_keyboard(self, chat_id, text, keyboard_options):
        reply_markup = {
            "keyboard": keyboard_options,
            "resize_keyboard": True,
            "one_time_keyboard": True
        }
        self.send_message(chat_id, text, reply_markup=json.dumps(reply_markup))

    def modify_keyboard(self, chat_id, message_id, reply_markup):
        url = self.base_url + "editMessageReplyMarkup"
        data = {"chat_id": chat_id, "message_id": message_id, "reply_markup": json.dumps(reply_markup)}
        requests.post(url, data=data)

    def close_keyboard(self, chat_id, text="Keyboard closed"):
        reply_markup = {"remove_keyboard": True}
        self.send_message(chat_id, text, reply_markup=json.dumps(reply_markup))


    # Add more methods as needed for inline buttons, removing keyboards, handling commands, etc.
    def openai_chat(self, chat_id, message):
        """Generates a response using OpenAI and sends it back to the user, if enabled."""
        if not hasattr(self, 'openai_available') or not self.openai_available:
            self.send_message(chat_id, "OpenAI integration is not enabled.")
            return
        
        try:
            import openai
            response = openai.ChatCompletion.create(
                model=self.openai_model,
                messages=[
                    {"role": "system", "content": "You are a helpful assistant."},
                    {"role": "user", "content": message}
            ]
        )
            if response.choices:
                self.send_message(chat_id, response.choices[0].message['content'])
            else:
                self.send_message(chat_id, "I'm not sure how to respond to that.")
        except Exception as e:
            print(f"Error using OpenAI: {e}")
            self.send_message(chat_id, "Sorry, I couldn't process that. Please try again.")


    def modify_openai(self, model="gpt-3.5-turbo"):
        """Allows users to modify the OpenAI model used for chat responses."""
        self.openai_model = model

    def register_command(self, command, handler):
        """Register a command and its handler function."""
        self.command_handlers[command] = handler

    def handle_message(self, chat_id, text):
        """Default message handler."""
        if text.startswith("/") and text.split(" ")[0] in self.command_handlers:
            command = text.split(" ")[0]
            self.command_handlers[command](self, chat_id, text)
        elif text.startswith("/start"):
            self.send_message(chat_id, "Hello! I'm a Botte! Congratulations! for creating your new bot.")
        else:
            self.send_message(chat_id, "This is not a valid command")


    def get_updates(self, offset=None, timeout=30):
        url = self.base_url + "getUpdates"
        params = {"offset": offset, "timeout": timeout}
        response = requests.get(url, params=params)
        return response.json()
    
    def markdown_message(self, chat_id, markdown_text):
        """Sends a message with Markdown formatting converted to HTML."""
        html_text = parse_markdown(markdown_text)
        self.send_message(chat_id, html_text, parse_mode="HTML")

def runbot(token, message_handler):
    client = TelegramClient(token)
    client.listen_for_messages(message_handler)
    client.enable_openai_integration('your_openai_api_key')



from typing import Optional
from .markdown import parse_markdown, unparse_html

class Parser:
    def __init__(self):
        # Initializes the Parser; can be extended to include more parsers or configuration.
        pass

    def parse(self, text: str, mode: Optional[str] = "markdown"):
        """
        Parses text from Markdown to HTML or other formats depending on the mode.

        :param text: The input text to be parsed.
        :param mode: The parsing mode (e.g., "markdown" to HTML). Default is "markdown".
        :return: The parsed text.
        """
        text = text.strip()

        if mode == "markdown":
            return parse_markdown(text)
        elif mode is None or mode == "disabled":
            # Return text as is if parsing is disabled or not specified
            return text
        else:
            raise ValueError(f"Unsupported parse mode: {mode}")

    def unparse(self, text: str, mode: Optional[str] = "html"):
        """
        Converts text from HTML back to Markdown or other formats depending on the mode.

        :param text: The input HTML text to be unparsed.
        :param mode: The unparse mode (e.g., "html" to Markdown). Default is "html".
        :return: The unparsed text.
        """
        if mode == "html":
            return unparse_html(text)
        elif mode is None or mode == "disabled":
            # Return text as is if unparsing is disabled or not specified
            return text
        else:
            raise ValueError(f"Unsupported unparse mode: {mode}")
